/*
 * crypto/testing/testcip.c
 *
 * Written by Alexander Kjeldaas <astor@guardian.no> 1998-12-17
 *
 * Copyright 1998 by Alexander Kjeldaas. Redistribution of this file
 * is permitted under the GNU Public License.
 */

#define __KERNEL__

#include <linux/crypto.h>

#undef memcpy
#undef memset

#include "libcfuncs.h"
#include "misc.h"

static void show_usage (void);

static int plaintext[512];
static int ciphertext[512];
static int testres[512];
static char key[512];
static int plaintextlen, ciphertextlen, keylen;

int main(int argc, char **argv)
{
  struct cipher_implementation *ci = NULL;
  struct cipher_context *cx;
  int arg, err;
  int swap_to_processor = 0;
  int rc = 0;

  enum arg_flags {
    ARG_NONE = 0,
    ARG_C = 1 << 0,
    ARG_P = 1 << 1,
    ARG_E = 1 << 2,
    ARG_K = 1 << 3
  } given_args = ARG_NONE;

  /* Register all ciphers */
  init_ciphers_and_digests();

  memset (plaintext, 0, sizeof(plaintext));
  memset (ciphertext, 0, sizeof(ciphertext));
  memset (testres, 0, sizeof(testres));
  memset (key, 0, sizeof(key));

  while((arg = getopt(argc,argv,"hp:e:c:k:s")) != EOF) {
    switch(arg) {
    case 'h': 
      show_usage(); 
      break;
    case 'c': 
      if (!(ci = find_cipher_by_name(optarg, 1))) 
	{
	  printf ("no cipher by that name (%s)\n", optarg);
	  exit (1);
	}
      given_args |= ARG_C; 
      break;
    case 'p': 
      plaintextlen = parsehex(optarg, (char *)&plaintext); 
      given_args |= ARG_P;
      break;
    case 'e': 
      ciphertextlen = parsehex(optarg, (char *)&ciphertext); 
      given_args |= ARG_E;
      break;
    case 'k': 
      keylen = parsehex(optarg, key);
      given_args |= ARG_K;
      break;
    case 's': 
      swap_to_processor = 1;
      break;
    }
  }

  if (given_args != (ARG_C | ARG_P | ARG_E | ARG_K)) 
    show_usage();

  if (ciphertextlen != plaintextlen) {
    printf ("size of plaintext and ciphertext differ\n");
    exit(1);
  }

  if (swap_to_processor) {
    swab32_buf((char *)ciphertext, ciphertextlen);
  }

  cx = (struct cipher_context *) malloc(sizeof(struct cipher_context));
  cx->ci = ci;
  cx->keyinfo = (u32 *) malloc(ci->key_schedule_size);
  if ((err = ci->set_key(cx, (char *)key, keylen))) {
    printf ("set_key returned error %d\n", err);
    exit(1);
  }

  if ((err = ci->encrypt(cx, (u8 *)plaintext, (u8 *)testres, 
			 plaintextlen, 0))) {
    printf("encrypt returned error %d\n", err);
    exit(1);
  }


  if (memcmp(testres, ciphertext, ciphertextlen)) {
    printf("encrypted text and ciphertext differ\n");
    printf("generated ciphertext (%d bytes):\n", ciphertextlen);
    dump_buffer((char *)testres, ciphertextlen);
    printf("correct ciphertext (%d bytes):\n", ciphertextlen);
    dump_buffer((char *)ciphertext, ciphertextlen);
    rc = 2;
  }


  if ((err = ci->decrypt(cx, (u8 *)ciphertext, (u8 *)testres, 
			 ciphertextlen, 0))) {
    printf("decrypt returned error %d\n", err);
    exit(1);
  }

  if (memcmp(testres, plaintext, plaintextlen)) {
    printf("decrypted ciphertext and plaintext differ\n");
    printf("generated plaintext (%d bytes):\n", plaintextlen);
    dump_buffer((char *)testres, plaintextlen);
    printf("correct plaintext (%d bytes):\n", plaintextlen);
    dump_buffer((char *)plaintext, plaintextlen);
    rc = 2;
  }
  return rc;
}

static void show_usage(void) 
{
  printf("Usage: testcip [-h] -k <key> -c <cipher> -p <plaintext> -e <encryptedtext>\n");
  printf("       key, plaintext and encryptedtext are hex strings\n");
  exit(1);
}
	
